Package: mlr3spatial
Title: Support for Spatial Objects Within the 'mlr3' Ecosystem
Version: 0.7.0
Authors@R: 
    c(
     person("Marc", "Becker", , "marcbecker@posteo.de", role = c("aut", "cre"),
            comment = c(ORCID = "0000-0002-8115-0400")),
     person("Patrick", "Schratz", , "patrick.schratz@gmail.com", role = "aut",
            comment = c(ORCID = "0000-0003-0748-6624"))
    )
Description: Extends the 'mlr3' ML framework with methods for spatial
    objects. Data storage and prediction are supported for packages
    'terra', 'raster' and 'stars'.
License: LGPL-3
URL: https://mlr3spatial.mlr-org.com,
        https://github.com/mlr-org/mlr3spatial
BugReports: https://github.com/mlr-org/mlr3spatial/issues
Depends: mlr3 (>= 1.2.0), R (>= 3.1.0)
Imports: checkmate (>= 2.0.0), data.table (>= 1.14.0), lgr (>= 0.4.2),
        methods, mlr3misc (>= 0.11.0), R6 (>= 2.5.0), sf, terra (>=
        1.6-3), utils
Suggests: bench, future, future.callr, knitr, mlr3learners (>= 0.4.5),
        paradox, ranger, raster, rmarkdown, rpart, stars (>= 0.5-5),
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: false
Encoding: UTF-8
LazyData: true
Collate: 'DataBackendRaster.R' 'DataBackendVector.R'
        'LearnerClassifSpatial.R' 'LearnerRegrSpatial.R' 'TaskRegrST.R'
        'TaskClassifST.R' 'TaskClassif_leipzig.R'
        'as_task_classif_st.R' 'as_task_regr_st.R'
        'as_task_unsupervised.R' 'data.R' 'helper.R'
        'predict_spatial.R' 'zzz.R'
Config/roxygen2/version: 8.0.0.9000
NeedsCompilation: no
Packaged: 2026-07-14 12:26:01 UTC; marc
Author: Marc Becker [aut, cre] (ORCID: <https://orcid.org/0000-0002-8115-0400>),
  Patrick Schratz [aut] (ORCID: <https://orcid.org/0000-0003-0748-6624>)
Maintainer: Marc Becker <marcbecker@posteo.de>
Repository: CRAN
Date/Publication: 2026-07-14 12:50:02 UTC
