# Generated by roxygen2: do not edit by hand

S3method(as_data_backend,RasterBrick)
S3method(as_data_backend,RasterStack)
S3method(as_data_backend,SpatRaster)
S3method(as_data_backend,sf)
S3method(as_data_backend,stars)
S3method(as_task_classif_st,DataBackend)
S3method(as_task_classif_st,TaskClassifST)
S3method(as_task_classif_st,TaskRegrST)
S3method(as_task_classif_st,data.frame)
S3method(as_task_classif_st,sf)
S3method(as_task_regr_st,DataBackend)
S3method(as_task_regr_st,TaskClassifST)
S3method(as_task_regr_st,TaskRegrST)
S3method(as_task_regr_st,data.frame)
S3method(as_task_regr_st,sf)
S3method(as_task_unsupervised,RasterBrick)
S3method(as_task_unsupervised,RasterStack)
S3method(as_task_unsupervised,SpatRaster)
S3method(as_task_unsupervised,sf)
S3method(as_task_unsupervised,stars)
export(DataBackendRaster)
export(DataBackendVector)
export(TaskClassifST)
export(TaskRegrST)
export(as_data_backend.RasterBrick)
export(as_data_backend.RasterStack)
export(as_data_backend.SpatRaster)
export(as_data_backend.sf)
export(as_data_backend.stars)
export(as_task_classif_st)
export(as_task_classif_st.DataBackend)
export(as_task_classif_st.TaskClassifST)
export(as_task_classif_st.TaskRegrST)
export(as_task_classif_st.data.frame)
export(as_task_classif_st.sf)
export(as_task_regr_st)
export(as_task_regr_st.DataBackend)
export(as_task_regr_st.TaskClassifST)
export(as_task_regr_st.TaskRegrST)
export(as_task_regr_st.data.frame)
export(as_task_regr_st.sf)
export(block_size)
export(factor_layer)
export(generate_stack)
export(mask_stack)
export(numeric_layer)
export(predict_spatial)
export(sample_stack)
export(weights_layer)
export(write_raster)
import(checkmate)
import(data.table)
import(mlr3)
import(mlr3misc)
import(sf)
importFrom(R6,R6Class)
importFrom(R6,is.R6)
importFrom(methods,as)
importFrom(stats,complete.cases)
importFrom(stats,runif)
importFrom(terra,cats)
importFrom(terra,head)
importFrom(terra,intersect)
importFrom(terra,ncell)
importFrom(terra,ncol)
importFrom(terra,nlyr)
importFrom(terra,rast)
importFrom(terra,readStart)
importFrom(terra,readStop)
importFrom(terra,readValues)
importFrom(terra,rowColFromCell)
importFrom(terra,sources)
importFrom(terra,unique)
importFrom(terra,writeRaster)
importFrom(terra,writeStart)
importFrom(terra,writeStop)
importFrom(utils,data)
importFrom(utils,getFromNamespace)
importFrom(utils,tail)
